<?php
namespace Opencart\Admin\Controller\Extension\Chatbro\Module;

//for 4.0.0.0
require_once DIR_EXTENSION . '/chatbro/system/library/library_autoloader.php';

use \Opencart\System\Library\Extension\Chatbro\LibraryloaderTrait;
use \Opencart\System\Library\Extension\Chatbro\StoreloaderTrait;
use \Opencart\System\Library\Extension\Chatbro\Log;

class Chatbro extends \Opencart\System\Engine\Controller
{
    use StoreloaderTrait;
    use LibraryloaderTrait;

    private $debug_log;
    const DEBUG_LOG_FILENAME = 'chatbro.%s.log';

    private $store_id = 0;

    public function __construct(\Opencart\System\Engine\Registry $registry)
    {
        parent::__construct($registry);

        $this->config->addPath('extension/chatbro', DIR_EXTENSION . 'chatbro/system/config');

        $this->store_id = isset($this->request->get['store_id']) ? (int) $this->request->get['store_id'] : 0;
        $this->debug_log = new Log(sprintf(self::DEBUG_LOG_FILENAME, $this->store_id));

        $this->load->model('extension/chatbro/proxy/url');

        $this->loadStore($this->store_id);
        $this->loadLibrary($this->store_id);
    }

    public function index(): void
    {
        $this->load->language('extension/chatbro/module/chatbro');
        $this->document->setTitle($this->language->get('heading_title'));

        $this->document->addStyle(HTTP_CATALOG . 'extension/chatbro/admin/view/stylesheet/chatbro/chatbro.css');
        $this->document->addStyle(HTTP_CATALOG . 'extension/chatbro/admin/view/stylesheet/chatbro/chatbro-bootstrap.css');

        $this->document->addScript(HTTP_CATALOG . 'extension/chatbro/admin/view/javascript/chatbro/chatbro.js');
        $this->document->addScript(HTTP_CATALOG . 'extension/chatbro/admin/view/javascript/chatbro/common.js');

        $this->initChatbro();

        $data['breadcrumbs'] = [];
        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'])
        ];
        $data['breadcrumbs'][] = [
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=module')
        ];
        $data['breadcrumbs'][] = [
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/chatbro/module/chatbro', 'user_token=' . $this->session->data['user_token'])
        ];

        $data['back'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=module');

        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $data['render_admin_block'] = $this->chatbroapp->admin();

        $this->response->setOutput($this->load->view('extension/chatbro/module/chatbro', $data));
    }

    protected function initChatbro()
    {
        $this->chatbroapp->init();
    }

    public function get_chats(): void
    {
        $this->initChatbro();
        $json = $this->chatbroapp->getChats();

        $this->response->addHeader('Content-Type: text/html; charset=utf-8');
        $this->response->setOutput($json);
    }

    public function delete_chat(): void
    {
        $this->initChatbro();
        $json = $this->chatbroapp->deleteChat();

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    public function update_chat(): void
    {
        $this->initChatbro();
        $json = $this->chatbroapp->updateChat();

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    public function save_settings(): void
    {

        $this->initChatbro();
        $this->load->model('extension/chatbro/module/chatbro');
        $json = $this->chatbroapp->saveSettings();

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    public function create_chat(): void
    {
        $this->initChatbro();
        $this->load->model('extension/chatbro/module/chatbro');
        $json = $this->chatbroapp->createChat();

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    public function view_column_left(string $route, array &$data): void
    {
        $chatbro_menu = [
            'id' => 'menu-chatbro',
            'icon' => 'fa fa-cog',
            'name' => 'Chatbro Extension',
            'href' => $this->url->link('extension/chatbro/module/chatbro', 'user_token=' . $this->session->data['user_token'], true),
            'children' => []
        ];

        array_unshift($data['menus'], $chatbro_menu);
    }

    public function install(): void
    {
        $this->load->model('extension/chatbro/module/chatbro');
        $this->model_extension_chatbro_module_chatbro->createTables();
        $this->model_extension_chatbro_module_chatbro->createEvents();
    }

    public function uninstall(): void
    {
        $this->load->model('extension/chatbro/module/chatbro');
        $this->model_extension_chatbro_module_chatbro->dropTables();
        $this->model_extension_chatbro_module_chatbro->deleteEvents();
    }
}
